@@ -48,6 +48,9 @@ class DockerManager(object):
             'yarn': { 
                 'server' : self.config.yarn,
                 'client' : self.config.hadoop_client },
+            'spark': { 
+                'server' : self.config.spark,
+                'client' : self.config.spark_client },
             'gluster': { 
                 'server' : self.config.gluster },
             'cassandra': { 
@@ -488,10 +491,9 @@ class DockerManager(object):
     Read the directory containing the key we should use. 
     """
     def _read_key_dir(self):
-        f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
-        k = f.read().strip().split("://")
-        # return { k[1] : '/service/keys' }
-        return { '/service/keys' : k[1]  }
+        with open(ferry.install.DEFAULT_DOCKER_KEY, 'r') as f:
+            k = f.read().strip().split("://")
+            return { '/service/keys' : k[1]  }
 
     """
     Prepare the environment for storage containers.
@@ -625,10 +627,10 @@ class DockerManager(object):
         plan['localhost']['containers'].append(container_info)
         return plan
 
-    """
-    Transfer the configuration to the containers. 
-    """
     def _transfer_config(self, config_dirs):
+        """
+        Transfer the configuration to the containers. 
+        """
         for c in config_dirs:
             container = c[0]
             from_dir = c[1]
@@ -636,6 +638,17 @@ class DockerManager(object):
             logging.warning("transfer config %s -> %s" % (from_dir, to_dir))
             self.docker.copy([container], from_dir, to_dir)
 
+    def _transfer_ip(self, ips):
+        """
+        Transfer the hostname/IP addresses to all the containers. 
+        """
+        with open('/tmp/instances', 'w+') as hosts_file:
+            for ip in ips:
+                hosts_file.write("%s %s\n" % (ip[0], ip[1]))
+        for ip in ips:
+            self.docker.copy_raw(ip[0], '/tmp/instances', '/service/sconf/instances')
+            self.docker.cmd_raw(ip[0], '/service/sbin/startnode hosts')
+        
     """
     Transfer these environment variables to the containers.
     Since the user normally interacts with these containers by 
@@ -654,8 +667,30 @@ class DockerManager(object):
     """
     Restart the stopped containers. 
     """
-    def _restart_containers(self, container_info):
-        return self.docker.restart(container_info)
+    def _restart_containers(self, containers):
+        return self.docker.restart(containers)
+
+    def cancel_stack(self, backends, connectors):
+        """
+        The stack could not be instantiated correctly. Just get rid
+        of these containers. 
+        """
+        for b in backends['uuids']:
+            conf = self._get_service_configuration(b['storage'], detailed=True)
+            if conf and 'containers' in conf:
+                for c in conf['containers']:
+                    self.docker.stop([c])
+            for compute in b['compute']:
+                conf = self._get_service_configuration(compute, detailed=True)
+                if conf and 'containers' in conf:
+                    for c in conf['containers']:
+                        self.docker.stop([c])
+
+        for b in connectors:
+            s = self._get_service_configuration(b, detailed=True)
+            if s and 'containers' in s:
+                for c in s['containers']:
+                    self.docker.stop([c])
 
     """
     Register the set of services under a single cluster identifier. 
@@ -741,13 +776,22 @@ class DockerManager(object):
     Stop a running cluster.
     """
     def _stop_stack(self, cluster_uuid):
+        # First stop all the running services. 
         connectors, compute, storage = self._get_cluster_instances(cluster_uuid)
         for c in connectors:
             self._stop_service(c['uuid'], c['instances'], c['type'])
         for c in compute:
             self._stop_service(c['uuid'], c['instances'], c['type'])
-        for s in storage:
-            self._stop_service(s['uuid'], s['instances'], s['type'])
+        for c in storage:
+            self._stop_service(c['uuid'], c['instances'], c['type'])
+
+        # Then actually stop the containers. 
+        for c in connectors:
+            self.docker.halt(c['instances'])
+        for c in compute:
+            self.docker.halt(c['instances'])
+        for c in storage:
+            self.docker.halt(c['instances'])
 
     def _purge_stack(self, cluster_uuid):
         volumes = []
@@ -800,20 +844,12 @@ class DockerManager(object):
             self.cluster_collection.update( {'uuid':cluster_uuid}, 
                                              {"$set": cluster_state } )
 
-    """
-    Restart an stopped compute cluster.
-    """
-    def restart_compute(self, service_uuid, container_info, compute_type):
-        containers = self._restart_containers(container_info)
-        container_info = self._serialize_containers(containers)
-
-        service = {'uuid':service_uuid, 
-                   'containers':container_info, 
-                   'status':'running'}
-        self._update_service_configuration(service_uuid, service)
-
-        self._restart_service(service_uuid, containers, compute_type)
-        return service_uuid        
+    def start_service(self, uuid, containers):
+        """
+        Start the service.
+        """
+        service_info = self._get_service_configuration(uuid, detailed=True)
+        self._start_service(uuid, containers, service_info)
 
     """
     Allocate a new compute cluster.
@@ -863,18 +899,22 @@ class DockerManager(object):
 
         # After the docker instance start, we need to start the
         # actual storage service (gluster, etc.). 
-        self._start_service(service_uuid, containers, compute_type, entry_point)
-        return service_uuid
+        # self._start_service(service_uuid, containers, compute_type, entry_point)
+        return service_uuid, containers
         
     def _start_service(self,
                        uuid,
                        containers,
-                       service_type,
-                       entry_point, 
-                       service=None):
-        if not service:
-            service = self._get_service(service_type)
-        service.start_service(containers, entry_point, self.docker)
+                       service_info): 
+        if service_info['class'] != 'connector':
+            service = self._get_service(service_info['type'])
+            service.start_service(containers, service_info['entry'], self.docker)
+        else:
+            storage_entry = service_info['storage']
+            compute_entry = service_info['compute']
+            services, backend_names = self._get_client_services(storage_entry, compute_entry)
+            for service in services:
+                service.start_service(containers, service_info['entry'], self.docker)
 
     def _restart_service(self,
                          uuid,
@@ -905,20 +945,20 @@ class DockerManager(object):
                 if 'client' in self.service[backend]:
                     service = self.service[backend]['client']
                     service.stop_service(containers, entry_point, self.docker)
-    """
-    Restart an stopped storage cluster.
-    """
-    def restart_storage(self, service_uuid, container_info, storage_type):
-        containers = self._restart_containers(container_info)
+        
+    def restart_containers(self, service_uuid, containers):
+        """
+        Restart an stopped storage cluster. This does not
+        re-initialize the container. It just starts an empty
+        container. 
+        """
+        self._restart_containers(containers)
         container_info = self._serialize_containers(containers)
 
         service = {'uuid':service_uuid, 
                    'containers':container_info, 
                    'status':'running'}
         self._update_service_configuration(service_uuid, service)
-
-        self._restart_service(service_uuid, containers, storage_type)
-        return service_uuid        
                         
     """
     Create a storage cluster and start a particular
@@ -958,11 +998,7 @@ class DockerManager(object):
                    'entry':entry_point,
                    'status':'running'}
         self._update_service_configuration(service_uuid, service)
-
-        # After the docker instance start, we need to start the
-        # actual storage service (gluster, etc.). 
-        self._start_service(service_uuid, containers, storage_type, entry_point)
-        return service_uuid
+        return service_uuid, containers
 
     """
     Get the default deployment conf file. 
@@ -980,14 +1016,13 @@ class DockerManager(object):
 
         # Read the configuration file.
         if os.path.exists(conf):
-            f = open(conf, 'r').read()
-            j = json.loads(f)
-
-            # Now find the right configuration.
-            if mode in j:
-                j[mode]['_mode'] = mode
-                return j[mode]
+            with open(conf, 'r') as f: 
+                j = json.loads(f.read())
 
+                # Now find the right configuration.
+                if mode in j:
+                    j[mode]['_mode'] = mode
+                    return j[mode]
         return None
 
     """
@@ -1072,6 +1107,7 @@ class DockerManager(object):
     def fetch_snapshot_backend(self, snapshot_uuid):
         snapshot = self.cluster_collection.find_one( {'snapshot_uuid':snapshot_uuid} )
         if snapshot:
+            logging.warning("SNAPSHOT BACKEND: " + str(snapshot['backends']))
             return snapshot['backends']['backend']
 
     """
@@ -1093,6 +1129,7 @@ class DockerManager(object):
                                      backend_info,
                                      conf = None):
         connector_info = []
+        connector_plan = []
         cluster = self.cluster_collection.find_one( {'uuid':app_uuid} )
         if cluster:
             for cuid in cluster['connectors']:
@@ -1100,8 +1137,15 @@ class DockerManager(object):
                 # contain the container ID. 
                 s = self._get_service_configuration(cuid, detailed=True)
                 if s and 'containers' in s:
-                    connector_info.append(self._restart_connectors(cuid, s['containers'], backend_info))
-        return connector_info
+                    containers = [DockerInstance(j) for j in s['containers']]
+                    connector_plan.append( { 'uuid' : cuid,
+                                             'containers' : containers,
+                                             'type' : s['type'], 
+                                             'backend' : backend_info, 
+                                             'start' : 'restart' } )
+                    connector_info.append(cuid)
+                    self.restart_containers(cuid, containers)
+        return connector_info, connector_plan
 
     """
     Lookup the deployed application connector info and instantiate. 
@@ -1111,18 +1155,24 @@ class DockerManager(object):
                                      backend_info,
                                      conf = None):
         connector_info = []
+        connector_plan = []
         app = self.deploy.find( one = True,
                                 spec = { 'uuid' : app_uuid },
                                 conf = conf)
         if app:
             for c in app['connectors']:
-                connector_info.append(self.allocate_connector(connector_type = c['type'],
-                                                              backend = backend_info,
-                                                              name = c['name'], 
-                                                              args = c['args'],
-                                                              ports = s['ports'].keys(),
-                                                              image = c['image']))
-        return connector_info
+                uuid, containers = self.allocate_connector(connector_type = c['type'],
+                                                           backend = backend_info,
+                                                           name = c['name'], 
+                                                           args = c['args'],
+                                                           ports = c['ports'].keys(),
+                                                           image = c['image'])
+                connector_info.append(uuid)
+                connector_plan.append( { 'uuid' : uuid,
+                                         'containers' : containers,
+                                         'type' : c['type'], 
+                                         'start' : 'start' } )
+        return connector_info, connector_plan
                 
     """
     Lookup the snapshot connector info and instantiate. 
@@ -1131,16 +1181,22 @@ class DockerManager(object):
                                      snapshot_uuid, 
                                      backend_info):
         connector_info = []
+        connector_plan = []
         snapshot = self.snapshot_collection.find_one( {'snapshot_uuid':snapshot_uuid} )
         if snapshot:
             for s in snapshot['snapshot_cs']:
-                connector_info.append(self.allocate_connector(connector_type = s['type'],
-                                                              backend = backend_info,
-                                                              name = s['name'], 
-                                                              args = s['args'],
-                                                              ports = s['ports'].keys(),
-                                                              image = s['image']))
-        return connector_info
+                uuid, containers = self.allocate_connector(connector_type = s['type'],
+                                                           backend = backend_info,
+                                                           name = s['name'], 
+                                                           args = s['args'],
+                                                           ports = s['ports'].keys(),
+                                                           image = s['image'])
+                connector_info.append(uuid)
+                connector_plan.append( { 'uuid' : uuid,
+                                         'containers' : containers,
+                                         'type' : s['type'], 
+                                         'start' : 'start' } )
+        return connector_info, connector_plan
                 
 
     """
@@ -1148,7 +1204,7 @@ class DockerManager(object):
     """
     def _restart_connectors(self,
                             service_uuid, 
-                            container_info, 
+                            connectors, 
                             backend=None):
         # Initialize the connector and connect to the storage. 
         storage_entry = []
@@ -1160,11 +1216,6 @@ class DockerManager(object):
                 for c in b['compute']:
                     compute_entry.append(self._get_service_configuration(c))
 
-        # Start the connectors.
-        connector_type = container_info[0]['type']
-        args = container_info[0]['args']
-        connectors = self._restart_containers(container_info)
-
         # Generate the environment variables that will be 
         # injected into the containers. 
         env_vars = self.config.generate_env_vars(storage_entry,
@@ -1177,7 +1228,7 @@ class DockerManager(object):
                                                                                     service,
                                                                                     storage_entry,
                                                                                     compute_entry,
-                                                                                    args)
+                                                                                    connectors[0].args)
             # Merge all the entry points. 
             entry_points = dict(entry_point.items() + entry_points.items())
 
@@ -1186,7 +1237,7 @@ class DockerManager(object):
             self._transfer_env_vars(connectors, env_vars)
 
         # Start the containers and update the state. 
-        self._restart_service(service_uuid, connectors, connector_type)
+        self._restart_service(service_uuid, connectors, connectors[0].service_type)
         container_info = self._serialize_containers(connectors)
         service = {'uuid':service_uuid, 
                    'containers':container_info, 
@@ -1194,7 +1245,6 @@ class DockerManager(object):
                    'entry':entry_points,
                    'status':'running'}
         self._update_service_configuration(service_uuid, service)
-        return service_uuid
                 
     """
     Allocate a new connector and associate with an existing storage service. 
@@ -1237,7 +1287,6 @@ class DockerManager(object):
         entry_points = {}
         services, backend_names = self._get_client_services(storage_entry, compute_entry)
         for service in services:
-            logging.warning("USING CONNECTOR SERVICE: " + str(service))
             config_dirs, entry_point = self.config.generate_connector_configuration(service_uuid, 
                                                                                     containers, 
                                                                                     service,
@@ -1251,21 +1300,20 @@ class DockerManager(object):
             self._transfer_config(config_dirs)
             self._transfer_env_vars(containers, env_vars)
 
-            # Start the connector personality. 
-            self._start_service(service_uuid, containers, connector_type, entry_point, service)
-
         # Update the connector state. 
         container_info = self._serialize_containers(containers)
         service_info = {'uuid':service_uuid, 
                         'containers':container_info, 
                         'backends':backend_names, 
+                        'storage': storage_entry, 
+                        'compute': compute_entry, 
                         'class':'connector',
                         'type':connector_type,
                         'entry':entry_points,
                         'uniq': name, 
                         'status':'running'}
         self._update_service_configuration(service_uuid, service_info)
-        return service_uuid
+        return service_uuid, containers
 
     """
     Fetch the current docker version.
