@@ -16,44 +16,16 @@
 import json
 import logging
 from flask import Flask, request
+from ferry.install import Installer
 from ferry.docker.manager import DockerManager
+from ferry.docker.docker import DockerInstance
 
 # Initialize Flask
 app = Flask(__name__)
 
 # Initialize the storage driver
 docker = DockerManager()
-
-"""
-Generate a filesystem status in JSON format. 
-"""
-class AllocationResponse(object):
-    # Codes to indicate status
-    NOT_EXISTS    = 0
-    BUILDING      = 1
-    READY         = 2
-    SHUTTING_DOWN = 3
-    SHUT_DOWN     = 4
-    TERMINATING   = 5
-    TERMINATED    = 6
-
-    def __init__(self):
-        self.uuid = None
-        self.status = self.NOT_EXISTS
-        self.num_instances = 0
-        self.type = None
-
-    """
-    Return the JSON response. 
-    """
-    def json(self):
-        json_data = {}
-        # json_data['uuid'] = str(self.uuid)
-        # json_data['status'] = str(self.status)
-        # json_data['instances'] = str(self.num_instances)
-        # json_data['type'] = str(self.num_instances)
-
-        return json.dumps(json_data, sort_keys=True)
+installer = Installer()
 
 """
 Fetch the current information for a particular filesystem. 
@@ -87,8 +59,10 @@ Allocate the backend from a snapshot.
 def _allocate_backend_from_snapshot(payload):
     snapshot_uuid = payload['_file']
     backends = docker.fetch_snapshot_backend(snapshot_uuid)
-    return _allocate_backend(payload = None,
-                             backends = backends)
+
+    if backends:
+        return _allocate_backend(payload = None,
+                                 backends = backends)
 
 """
 Allocate the backend from a deployment. 
@@ -147,12 +121,12 @@ def _allocate_compute(computes, storage_uuid, options=None):
     of all UUIDs that were created in the process. 
     """
     uuids = []
+    compute_plan = []
     for c in computes:
         compute_type = c['personality']
         reply = _fetch_num_instances(c['instances'], compute_type, options)
         num_instances = reply['num']
-        logging.warning("using %d instances (%s)" % (num_instances, compute_type))
-
+        c['instances'] = num_instances
         args = {}
         if 'args' in c:
             args = c['args']
@@ -161,13 +135,17 @@ def _allocate_compute(computes, storage_uuid, options=None):
         if 'layers' in c:
             layers = c['layers']
 
-        compute_uuid = docker.allocate_compute(compute_type = compute_type,
-                                               storage_uuid = storage_uuid, 
-                                               args = args, 
-                                               num_instances = num_instances,
-                                               layers = layers)
-        uuids.append(compute_uuid)
-    return uuids
+        compute_uuid, compute_containers = docker.allocate_compute(compute_type = compute_type,
+                                                                   storage_uuid = storage_uuid, 
+                                                                   args = args, 
+                                                                   num_instances = num_instances,
+                                                                   layers = layers)
+        compute_plan.append( { 'uuid' : compute_uuid,
+                               'containers' : compute_containers,
+                               'type' : compute_type, 
+                               'start' : 'start' } )
+        uuids.append( compute_uuid )
+    return uuids, compute_plan
 
 def _query_backend_params(backends, options=None):
     """
@@ -186,14 +164,22 @@ def _query_backend_params(backends, options=None):
 
 def _restart_compute(computes):
     uuids = []
+    compute_plan = []
     for c in computes:
         service_uuid = c['uuid']
-        containers = c['containers']
         compute_type = c['type']
-        uuids.append(docker.restart_compute(service_uuid,
-                                            containers, 
-                                            compute_type))
-    return uuids
+
+        # Transform the containers into proper container objects.
+        compute_containers = c['containers']
+        containers = [DockerInstance(j) for j in compute_containers] 
+
+        uuids.append(service_uuid)
+        compute_plan.append( { 'uuid' : service_uuid,
+                               'containers' : containers,
+                               'type' : compute_type, 
+                               'start' : 'restart' } )
+        docker.restart_containers(service_uuid, containers)
+    return uuids, compute_plan
 
 """
 Allocate a brand new backend
@@ -202,14 +188,14 @@ def _allocate_backend(payload,
                       backends=None,
                       replace=False,
                       uuid=None):
+    iparams = None
     if not backends:
         # The 'iparams' specifies the number of instances to use
         # for the various stack components. It is optional since
         # the application stack may already specify these values. 
-        iparams = None
         if 'iparams' in payload:
             iparams = payload['iparams']
-
+            
         # We should find the backend information in the payload. 
         if 'backend' in payload:
             backends = payload['backend']
@@ -223,6 +209,7 @@ def _allocate_backend(payload,
     backend_info = { 'status' : 'ok', 
                      'uuids' : [],
                      'backend' : backends }
+    storage_plan = []
 
     # First we need to check if either the storage or the compute 
     # have unspecified number of instances. If so, we'll need to abort
@@ -242,7 +229,7 @@ def _allocate_backend(payload,
         if len(all_questions) > 0:
             backend_info['status'] = 'query'
             backend_info['query'] = all_questions
-            return backend_info
+            return backend_info, None
     
     # Go ahead and create the actual backend stack. If the user has passed in
     # an existing backend UUID, that means we should restart that backend. Otherwise
@@ -256,42 +243,61 @@ def _allocate_backend(payload,
             storage_type = storage['personality']
             reply = _fetch_num_instances(storage['instances'], storage_type, iparams)
             num_instances = reply['num']
-            logging.warning("using %d instances (%s)" % (num_instances, storage_type))
+            storage['instances'] = num_instances
             layers = []
             if 'layers' in storage:
                 layers = storage['layers']
-            storage_uuid = docker.allocate_storage(storage_type = storage_type, 
-                                                   num_instances = num_instances,
-                                                   layers = layers,
-                                                   args = args,
-                                                   replace = replace)
+            storage_uuid, storage_containers = docker.allocate_storage(storage_type = storage_type, 
+                                                                       num_instances = num_instances,
+                                                                       layers = layers,
+                                                                       args = args,
+                                                                       replace = replace)
+            storage_plan.append( { 'uuid' : storage_uuid,
+                                   'containers' : storage_containers,
+                                   'type' : storage_type, 
+                                   'start' : 'start' } )
         else:
-            service_uuid = storage['uuid']
-            containers = storage['containers']
+            storage_uuid = storage['uuid']
             storage_type = storage['type']
-            storage_uuid = docker.restart_storage(service_uuid,
-                                                  container_info = containers,
-                                                  storage_type = storage_type)
+            storage_containers = storage['containers']
+
+            # Transform the containers into proper container objects.
+            containers = [DockerInstance(j) for j in storage_containers]
+            storage_plan.append( { 'uuid' : storage_uuid,
+                                   'containers' : containers,
+                                   'type' : storage_type, 
+                                   'start' : 'restart' } )
+            docker.restart_containers(storage_uuid, containers)
                                                   
         # Now allocate the compute backend. The compute is optional so
         # we should check if it even exists first. 
         compute_uuids = []
+        compute_plan = []
         if 'compute' in b:
             if not uuid:
-                compute_uuids = _allocate_compute(b['compute'], storage_uuid, iparams)
+                compute_uuids, compute_plan = _allocate_compute(b['compute'], storage_uuid, iparams)
             else:
-                compute_uuids = _restart_compute(b['compute'])
-
+                compute_uuids, compute_plan = _restart_compute(b['compute'])
+        
         backend_info['uuids'].append( {'storage':storage_uuid,
                                        'compute':compute_uuids} )
-    return backend_info
+    return backend_info, { 'storage' : storage_plan,
+                           'compute' : compute_plan }
 
 def _allocate_connectors(payload, backend_info):
     connector_info = []
+    connector_plan = []
     if 'connectors' in payload:
         connectors = payload['connectors']
         for c in connectors:
             connector_type = c['personality']
+            
+            # Check if this connector type has already been pulled
+            # into the local index. If not, manually pull it. 
+            if not installer._check_and_pull_image(connector_type):
+                # We could not fetch this connetor. Instead of 
+                # finishing, just return an error.
+                return False, connector_info, None
 
             # Connector names are created by the user 
             # to help identify particular instances. 
@@ -307,12 +313,17 @@ def _allocate_connectors(payload, backend_info):
             if 'ports' in c:
                 ports = c['ports']
 
-            connector_info.append(docker.allocate_connector(connector_type = connector_type,
-                                                            backend = backend_info, 
-                                                            name = connector_name, 
-                                                            args = args,
-                                                            ports = ports))
-    return connector_info
+            uuid, containers = docker.allocate_connector(connector_type = connector_type,
+                                                         backend = backend_info, 
+                                                         name = connector_name, 
+                                                         args = args,
+                                                         ports = ports)
+            connector_plan.append( { 'uuid' : uuid,
+                                     'containers' : containers,
+                                     'type' : connector_type, 
+                                     'start' : 'start' } )
+            connector_info.append(uuid)
+    return True, connector_info, connector_plan
 
 """
 Allocate the connectors from a snapshot. 
@@ -340,19 +351,89 @@ def _allocate_connectors_from_stopped(payload, backend_info, params=None):
                                               backend_info,
                                               params)
 
+def _register_ip_addresses(backend_plan, connector_plan):
+    """
+    Helper function to register the hostname/IP addresses
+    of all the containers. 
+    """
+    ips = []
+    for s in backend_plan['storage']:
+        for c in s['containers']:
+            if isinstance(c, dict):
+                ips.append( [c['internal_ip'], c['hostname']] )
+            else:
+                ips.append( [c.internal_ip, c.host_name] )
+    for s in backend_plan['compute']:
+        for c in s['containers']:
+            if isinstance(c, dict):
+                ips.append( [c['internal_ip'], c['hostname']] )
+            else:
+                ips.append( [c.internal_ip, c.host_name] )
+    for s in connector_plan:
+        for c in s['containers']:
+            # This is slightly awkward. It is because when starting
+            # a new stack, we get proper "container" objects. However,
+            # when restarting we get dictionary descriptions. Should just
+            # fix at the restart level! 
+            if isinstance(c, dict):
+                logging.warning("CONTAINER: " + str(c))
+                ips.append( [c['internal_ip'], c['hostname']] )
+            else:
+                ips.append( [c.internal_ip, c.host_name] )
+    docker._transfer_ip(ips)
+
+def _start_all_services(backend_plan, connector_plan):
+    """
+    Helper function to start both the backend and
+    frontend. Depending on the plan, this will either
+    do a fresh start or a restart on an existing cluster. 
+    """
+
+    # Make sure that all the hosts have the current set
+    # of IP addresses. 
+    _register_ip_addresses(backend_plan, connector_plan)
+
+    # Now we need to start/restart all the services. 
+    for s in backend_plan['storage']:
+        if s['start'] == 'start':
+            docker.start_service(s['uuid'], 
+                                 s['containers'])
+        else:
+            docker._restart_service(s['uuid'], s['containers'], s['type'])
+
+    for c in backend_plan['compute']:
+        if c['start'] == 'start':
+            docker.start_service(c['uuid'], c['containers'])
+        else:
+            docker._restart_service(c['uuid'], c['containers'], c['type'])
+
+    for c in connector_plan:
+        if c['start'] == 'start':
+            docker.start_service(c['uuid'], c['containers'])
+        else:
+            docker._restart_connectors(c['uuid'], c['containers'], c['backend'])
+
 def _allocate_new(payload):
     """
     Helper function to allocate and start a new stack. 
     """
     reply = {}
-    backend_info = _allocate_backend(payload, replace=True)
+    backend_info, backend_plan = _allocate_backend(payload, replace=True)
+    reply['status'] = backend_info['status']
+    logging.warning("NEW BACKEND: " + str(backend_info))
     if backend_info['status'] == 'ok':
-        connector_info = _allocate_connectors(payload, backend_info['uuids'])
-        uuid = docker.register_stack(backend_info, connector_info, payload['_file'])
-        reply['text'] = str(uuid)
+        success, connector_info, connector_plan = _allocate_connectors(payload, backend_info['uuids'])
+
+        if success:
+            _start_all_services(backend_plan, connector_plan)
+            uuid = docker.register_stack(backend_info, connector_info, payload['_file'])
+            reply['text'] = str(uuid)
+        else:
+            # One or more connectors was not instantiated properly. 
+            docker.cancel_stack(backend_info, connector_info)
+            reply['status'] = 'failed'
     else:
         reply['questions'] = backend_info['query']
-    reply['status'] = backend_info['status']
     return json.dumps(reply)
 
 def _allocate_stopped(payload):
@@ -360,38 +441,45 @@ def _allocate_stopped(payload):
     Helper function to allocate and start a stopped stack. 
     """
     base = docker.get_base_image(payload['_file'])
-    backend_info = _allocate_backend_from_stopped(payload)
+    backend_info, backend_plan = _allocate_backend_from_stopped(payload)
     if backend_info['status'] == 'ok':
-        connector_info = _allocate_connectors_from_stopped(payload, backend_info['uuids'])
-        
+        connector_info, connector_plan = _allocate_connectors_from_stopped(payload, backend_info['uuids'])
+        _start_all_services(backend_plan, connector_plan)        
         uuid = docker.register_stack(backends = backend_info,
                                      connectors = connector_info,
                                      base = base,
                                      uuid = payload['_file'])
         return json.dumps({'status' : 'ok',
                            'text' : str(uuid)})
+    else:
+        return json.dumps({'status' : 'failed'})
 
 def _allocate_snapshot(payload):
     """
     Helper function to allocate and start a snapshot.
     """
-    backend_info = _allocate_backend_from_snapshot(payload)
+    logging.warning("ALLOCATE SNAPSHOT " + str(payload['_file']))
+    backend_info, backend_plan = _allocate_backend_from_snapshot(payload)
     if backend_info['status'] == 'ok':
-        connector_info = _allocate_connectors_from_snapshot(payload, 
-                                                            backend_info['uuids'])
+        connector_info, connector_plan = _allocate_connectors_from_snapshot(payload, 
+                                                                            backend_info['uuids'])
+        _start_all_services(backend_plan, connector_plan)
         uuid = docker.register_stack(backend_info, connector_info, payload['_file'])
         return json.dumps({'status' : 'ok',
                            'text' : str(uuid)})
+    else:
+        return json.dumps({'status' : 'failed'})
 
 def _allocate_deployed(payload, params=None):
     """
     Helper function to allocate and start a deployed application. 
     """
-    backend_info = _allocate_backend_from_deploy(payload, params)
+    backend_info, backend_plan = _allocate_backend_from_deploy(payload, params)
     if backend_info['status'] == 'ok':
-        connector_info = _allocate_connectors_from_deploy(payload, 
-                                                          backend_info['uuids'],
-                                                          params)
+        connector_info, connector_plan = _allocate_connectors_from_deploy(payload, 
+                                                                          backend_info['uuids'],
+                                                                          params)
+        _start_all_services(backend_plan, connector_plan)
         uuid = docker.register_stack(backend_info, connector_info, payload['_file'])
         return json.dumps({'status' : 'ok',
                            'text' : str(uuid)})
