@@ -55,7 +55,7 @@ def _supported_arch():
 
 def _supported_lxc():
     output = Popen("(lxc-version 2>/dev/null || lxc-start --version) | sed 's/.* //'", stdout=PIPE, shell=True).stdout.read()
-    ver = tuple(map(int, (output.strip().split("."))))
+    ver = tuple(map(int, (output.strip().split(".")[:3])))
     return ver > (0, 7, 5)
 
 def _supported_python():
@@ -108,7 +108,7 @@ class Installer(object):
 
         # Only reset temporary keys. User-defined key directories
         # shouldn't be touched. 
-        if tmp == "tmp":
+        if keydir != DEFAULT_KEY_DIR and tmp == "tmp":
             shutil.rmtree(keydir)
 
         # Mark that we are using the default package keys
@@ -147,7 +147,7 @@ class Installer(object):
                 self._change_permission(DOCKER_DIR)
         except OSError as e:
             logging.error("Could not install Ferry.\n") 
-            logging.error(e.explanation)
+            logging.error(e.strerror)
             sys.exit(1)
 
         # Start the Ferry docker daemon. If it does not successfully
@@ -196,12 +196,18 @@ class Installer(object):
         not_installed = []
         images = ['mongodb', 'ferry-base', 'hadoop-base', 'hadoop', 'hadoop-client',
                   'hive-metastore', 'gluster', 'openmpi', 'cassandra', 'cassandra-client', 
-                  'titan']
+                  'titan', 'spark']
         for i in images:
             if not self._check_image_installed("%s/%s" % (DEFAULT_DOCKER_REPO, i)):
                 not_installed.append(i)
         return not_installed
 
+    def _check_and_pull_image(self, image_name):
+        if not self._check_image_installed(image_name):
+            self._pull_image(image_name, on_client=False)
+
+        return self._check_image_installed(image_name)
+
     def _check_image_installed(self, image_name):
         cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' inspect %s 2> /dev/null' % image_name
         output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
@@ -210,9 +216,11 @@ class Installer(object):
         else:
             return True
 
-    def start_web(self, options=None):
+    def start_web(self, options=None, clean=False):
         start, msg = self._start_docker_daemon(options)
-        if not start:
+        if not clean and not start:
+            # We are trying to start the web services but the Docker
+            # daemon won't start. If we're cleaning, it's not a big deal. 
             logging.error(msg) 
             sys.exit(1)
 
@@ -230,7 +238,7 @@ class Installer(object):
                 self._change_permission(DEFAULT_MONGO_LOG)
         except OSError as e:
             logging.error("Could not start ferry servers.\n") 
-            logging.error(e.explanation)
+            logging.error(e.strerror)
             sys.exit(1)
 
         # Check if the Mongo image is built.
@@ -374,6 +382,7 @@ class Installer(object):
         if self._docker_running():
             built_images = {}
             for f in os.listdir(image_dir):
+                logging.warning("transforming dockerfile")
                 self._transform_dockerfile(image_dir, f, repo)
 
             for f in os.listdir("/tmp/dockerfiles/"):
@@ -452,7 +461,6 @@ class Installer(object):
             dockerfile = image_dir + '/' + base + '/Dockerfile'
             self._build_image(base, dockerfile, repo, built_images, recurse, build)
 
-        # image = os.path.dirname(f).split("/")[-1]
         if not image in built_images:
             if base == "base":
                 self._pull_image(base, tag='latest')
@@ -480,35 +488,52 @@ class Installer(object):
                     return base[-1]
         return base
 
-    def _continuous_print(self, process):
+    def _continuous_print(self, process, on_client=True):
         while True:
-            out = process.stdout.read(15)
-            if out == '':
-                break
+            try:
+                out = process.stdout.read(15)
+                if out == '':
+                    break
+                else:
+                    if on_client:
+                        sys.stdout.write(out)
+                        sys.stdout.flush()
+                    else:
+                        logging.warning("downloading image...")
+            except IOError as e:
+                logging.warning(e)
+
+        try:
+            errmsg = process.stderr.readline()
+            if errmsg and errmsg != '':
+                logging.warning(errmsg)
             else:
-                sys.stdout.write(out)
-                sys.stdout.flush()
+                logging.warning("downloaded image!")
+        except IOError:
+            pass
 
-    def _pull_image(self, image, tag=None):
+    def _pull_image(self, image, tag=None, on_client=True):
         if not tag:
-            tag = ferry.__version__
-        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' pull %s:%s' % (image, tag)
-        logging.warning(cmd)
+            cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' pull %s' % image
+        else:
+            cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' pull %s:%s' % (image, tag)
 
-        child = Popen(cmd, stdout=PIPE, shell=True)
-        self._continuous_print(child)
+        logging.warning(cmd)
+        child = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
+        self._continuous_print(child, on_client=on_client)
 
         # Now tag the image with the 'latest' tag. 
-        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' tag' + ' %s:%s %s:%s' % (image, tag, image, 'latest')
-        logging.warning(cmd)
-        Popen(cmd, stdout=PIPE, shell=True)
+        if tag and tag != 'latest':
+            cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' tag' + ' %s:%s %s:%s' % (image, tag, image, 'latest')
+            logging.warning(cmd)
+            Popen(cmd, stdout=PIPE, shell=True)
         
     def _compile_image(self, image, repo, image_dir, build=False):
         # Now build the image. 
         if build:
             cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' build --rm=true -t' + ' %s/%s %s' % (repo, image, image_dir)
             logging.warning(cmd)
-            child = Popen(cmd, stdout=PIPE, shell=True)
+            child = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
             self._continuous_print(child)
 
             # Now tag the image. 
@@ -518,7 +543,7 @@ class Installer(object):
         else:
             # Just pull the image from the public repo. 
             image_name = "%s/%s" % (repo, image)
-            self._pull_image(image_name)
+            self._pull_image(image_name, tag=ferry.__version__)
 
     def _tag_images(self, image, repo, alternatives):
         for a in alternatives:
@@ -571,7 +596,7 @@ class Installer(object):
                 return False, "Ferry appears to be already running. If this is an error, please type \'ferry clean\' and try again."
         except OSError as e:
             logging.error("could not start docker daemon.\n") 
-            logging.error(e.explanation)
+            logging.error(e.strerror)
             sys.exit(1)
 
     def _stop_docker_daemon(self, force=False):
@@ -585,10 +610,10 @@ class Installer(object):
                 pass
 
     def _get_gateway(self):
-        cmd = "ifconfig drydock0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'"
+        cmd = "LC_MESSAGES=C ifconfig drydock0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'"
         gw = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
 
-        cmd = "ifconfig drydock0 | grep 'inet addr:' | cut -d: -f4 | awk '{ print $1}'"
+        cmd = "LC_MESSAGES=C ifconfig drydock0 | grep 'inet addr:' | cut -d: -f4 | awk '{ print $1}'"
         netmask = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
         mask = map(int, netmask.split("."))
         cidr = 1
